@@ -19,16 +19,22 @@
 #'   of all variables from a data frame, and due to computational reasons
 #'   character vectors should not be printed.
 #' @param show.na Logical, or \code{"auto"}. If \code{TRUE}, the output always
-#'   contains informatin on missing values, even if variables have no missing
+#'   contains information on missing values, even if variables have no missing
 #'   values. If \code{FALSE}, information on missing values are removed from
 #'   the output. If \code{show.na = "auto"}, information on missing values
 #'   is only shown when variables actually have missing values, else it's not
 #'   shown.
 #' @param grp.strings Numeric, if not \code{NULL}, groups string values in
-#'   character vectors, based on their similarity. The similarity is estimated
-#'   with the \pkg{stringdist}-package. See \code{\link{group_str}} for details
-#'   on grouping, and that function's \code{maxdist}-argument to get more
-#'   details on the distance of strings to be treated as equal.
+#'   character vectors, based on their similarity. See \code{\link{group_str}}
+#'   and \code{\link{str_find}} for details on grouping, and their
+#'   \code{precision}-argument to get more details on the distance of strings
+#'   to be treated as equal.
+#' @param min.frq Numeric, indicating the minimum frequency for which a
+#'   value will be shown in the output (except for the missing values, prevailing
+#'   \code{show.na}). Default value for \code{min.frq} is \code{0}, so all value
+#'   frequencies are shown. All values or categories that have less than
+#'   \code{min.frq} occurences in the data will be summarized in a \code{"n < 100"}
+#'   category.
 #' @param title String, will be used as alternative title to the variable
 #'   label. If \code{x} is a grouped data frame, \code{title} must be a
 #'   vector of same length as groups.
@@ -39,11 +45,16 @@
 #'   when \code{out} is not \code{"txt"}.
 #'
 #' @inheritParams descr
-#' @inheritParams to_factor
+#' @inheritParams to_dummy
 #'
 #' @return A list of data frames with values, value labels, frequencies, raw, valid and
 #'           cumulative percentages of \code{x}.
 #'
+#' @details The \dots-argument not only accepts variable names or expressions
+#'   from select-helpers. You can also use logical
+#'   conditions, math operations, or combining variables to produce "crosstables".
+#'   See 'Examples' for more details.
+#'
 #' @note \code{x} may also be a grouped data frame (see \code{\link[dplyr]{group_by}})
 #'       with up to two grouping variables. Frequency tables are created for each
 #'       subgroup then.
@@ -54,6 +65,9 @@
 #'       categorical variables (factors) or character vectors. In this case,
 #'       values are coerced into numeric vector to calculate the summary
 #'       statistics.
+#'       \cr \cr
+#'       To print tables in markdown or HTML format, use \code{print_md()} or
+#'       \code{print_html()}.
 #'
 #' @seealso \code{\link{flat_table}} for labelled (proportional) tables.
 #'
@@ -66,7 +80,14 @@
 #' library(dplyr)
 #' efc %>%
 #'   group_by(e16sex, c172code) %>%
-#'   frq(e16sex, c172code, e42dep)
+#'   frq(e42dep)
+#'
+#' # show only categories with a minimal amount of frequencies
+#' frq(mtcars$gear)
+#'
+#' frq(mtcars$gear, min.frq = 10)
+#'
+#' frq(mtcars$gear, min.frq = 15)
 #'
 #' # with select-helpers: all variables from the COPE-Index
 #' # (which all have a "cop" in their name)
@@ -80,6 +101,11 @@
 #' data(iris)
 #' frq(iris, Species)
 #'
+#' # also works on grouped data frames
+#' efc %>%
+#'   group_by(c172code) %>%
+#'   frq(is.na(nur_pst))
+#'
 #' # group variables with large range and with weights
 #' efc$weights <- abs(rnorm(n = nrow(efc), mean = 1, sd = .5))
 #' frq(efc, c160age, auto.grp = 5, weights = weights)
@@ -102,11 +128,29 @@
 #' frq(dummy)
 #' frq(dummy, grp.strings = 2)
 #'
-#' @importFrom stats na.omit
-#' @importFrom dplyr full_join select_if select
-#' @importFrom sjlabelled get_label get_labels get_values copy_labels
-#' @importFrom purrr map_if
-#' @importFrom rlang quo_name enquo
+#' #### other expressions than variables
+#'
+#' # logical conditions
+#' frq(mtcars, cyl ==6)
+#'
+#' frq(efc, is.na(nur_pst), contains("cop"))
+#'
+#' iris %>%
+#'   frq(starts_with("Petal"), Sepal.Length > 5)
+#'
+#' # computation of variables "on the fly"
+#' frq(mtcars, (gear + carb) / cyl)
+#'
+#' # crosstables
+#' set.seed(123)
+#' d <- data.frame(
+#'   var_x = sample(letters[1:3], size = 30, replace = TRUE),
+#'   var_y = sample(1:2, size = 30, replace = TRUE),
+#'   var_z = sample(LETTERS[8:10], size = 30, replace = TRUE)
+#' )
+#' table(d$var_x, d$var_z)
+#' frq(d, paste0(var_x, var_z))
+#' frq(d, paste0(var_x, var_y, var_z))
 #' @export
 frq <- function(x,
                 ...,
@@ -116,6 +160,7 @@ frq <- function(x,
                 show.strings = TRUE,
                 show.na = TRUE,
                 grp.strings = NULL,
+                min.frq = 0,
                 out = c("txt", "viewer", "browser"),
                 title = NULL,
                 encoding = "UTF-8",
@@ -128,8 +173,14 @@ frq <- function(x,
     out <- "txt"
   }
 
+  # check min.frq value
+  if (!is.numeric(min.frq)) {
+    message("min.frq value is not numeric. Returned output assumes default value 0.")
+    min.frq <- 0
+  }
+
   # get dot data
-  xw <- get_dot_data(x, dplyr::quos(...))
+  xw <- get_dot_data(x, dplyr::enquos(...))
 
   if (missing(weights)) {
     w <- NULL
@@ -156,17 +207,19 @@ frq <- function(x,
   }
 
 
-  # remove empty columns
+  if (!isTRUE(show.na)) {
+    # remove empty columns
 
-  rem.col <- empty_cols(x)
+    rem.col <- empty_cols(x)
 
-  if (!sjmisc::is_empty(rem.col)) {
-    rem.vars <- colnames(x)[rem.col]
-    x <- remove_empty_cols(x)
+    if (!sjmisc::is_empty(rem.col)) {
+      rem.vars <- colnames(x)[rem.col]
+      x <- remove_empty_cols(x)
 
-    message(sprintf("Following %i variables have only missing values and are not shown:", length(rem.vars)))
-    cat(paste(sprintf("%s [%i]", rem.vars, rem.col), collapse = ", "))
-    cat("\n")
+      message(sprintf("Following %i variables have only missing values and are not shown:", length(rem.vars)))
+      cat(paste(sprintf("%s [%i]", rem.vars, rem.col), collapse = ", "))
+      cat("\n")
+    }
   }
 
 
@@ -183,7 +236,8 @@ frq <- function(x,
   if (!show.strings)
     x <- dplyr::select_if(x, no_character)
 
-  if (sjmisc::is_empty(x)) return(NULL)
+  if ((all(sjmisc::is_empty(stats::na.omit(x), first.only = FALSE)) && show.na == FALSE) || all(suppressMessages(replace_na(sjmisc::is_empty(x, first.only = FALSE, all.na.empty = FALSE), value = FALSE))))
+    return(NULL)
 
 
   # group strings
@@ -200,7 +254,7 @@ frq <- function(x,
 
     x <- x %>%
       purrr::map_if(is.character, ~ group_str(
-        strings = .x, maxdist = grp.strings, remove.empty = FALSE)
+        strings = .x, precision = grp.strings, remove.empty = FALSE)
       ) %>%
       as.data.frame(stringsAsFactors = FALSE)
 
@@ -214,6 +268,11 @@ frq <- function(x,
   # do we have a grouped data frame?
   if (inherits(x, "grouped_df")) {
 
+    grkey <- colnames(dplyr::group_keys(x))
+    for (i in grkey) {
+      if (is.character(x[[i]])) x[[i]] <- as.factor(x[[i]])
+    }
+
     # get grouped data
     grps <- get_grouped_data(x)
 
@@ -247,10 +306,11 @@ frq <- function(x,
               cn = colnames(tmp)[1],
               auto.grp = auto.grp,
               title = gr.title,
-              show.na = show.na
+              show.na = show.na,
+              min.frq = min.frq
             )
 
-          attr(dummy, "group") <- get_grouped_title(x, grps, i, sep = "\n")
+          attr(dummy, "group") <- get_grouped_title(x, grps, i, sep = ", ", long = FALSE)
 
           # save data frame for return value
           dataframes[[length(dataframes) + 1]] <- dummy
@@ -279,7 +339,8 @@ frq <- function(x,
             cn = colnames(x)[i],
             auto.grp = auto.grp,
             title = title,
-            show.na = show.na
+            show.na = show.na,
+            min.frq = min.frq
           )
 
         # save data frame for return value
@@ -292,7 +353,7 @@ frq <- function(x,
   if (out == "txt")
     class(dataframes) <- c("sjmisc_frq", "list")
   else
-    class(dataframes) <- c("sjt_frq", "list")
+    class(dataframes) <- c("sjt_frq", "sjmisc_frq", "list")
 
   # save how to print output
   attr(dataframes, "print") <- out
@@ -303,18 +364,15 @@ frq <- function(x,
 }
 
 
-#' @importFrom dplyr n_distinct full_join bind_rows
-#' @importFrom stats na.omit xtabs na.pass sd weighted.mean
-#' @importFrom sjlabelled get_labels get_label as_numeric
-frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.na = TRUE) {
+frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.na = TRUE, min.frq = 0) {
   # remember type
   vartype <- var_type(x)
 
   # convert NaN and Inf to missing
   x <- zap_inf(x)
 
-  # variable with only mising?
-  if (length(stats::na.omit(x)) == 0) {
+  # variable with only missing?
+  if (length(stats::na.omit(x)) == 0 && show.na == FALSE) {
     mydat <- data.frame(
       val = NA,
       label = NA,
@@ -340,8 +398,8 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
 
     mean.value <- stats::weighted.mean(stats::na.omit(xnum), w = stats::na.omit(weight.by))
 
-    if (requireNamespace("sjstats", quietly = TRUE))
-      sd.value <- sjstats::wtd_sd(stats::na.omit(xnum), weights = stats::na.omit(weight.by))
+    if (requireNamespace("datawizard", quietly = TRUE))
+      sd.value <- datawizard::weighted_sd(stats::na.omit(xnum), weights = stats::na.omit(weight.by))
     else
       sd.value <- NA
 
@@ -371,9 +429,9 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
   labels <-
     sjlabelled::get_labels(
       x,
-      attr.only = T,
+      attr.only = TRUE,
       values = "n",
-      non.labelled = T
+      non.labelled = TRUE
     )
 
 
@@ -387,18 +445,17 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
   # do we have a labelled vector?
   if (!is.null(labels)) {
     # add rownames and values as columns
-    dat <-
-      data_frame(
-        n = names(labels),
-        v = as.character(labels)
-      )
+    dat <- data_frame(
+      n = names(labels),
+      v = as.character(labels)
+    )
 
     colnames(dat) <- c("val", "label")
 
     # character vectors need to be converted with to_value
     # to avoid NAs, but only if character is non-numeric
     if (is.character(dat$val) && anyNA(suppressWarnings(as.numeric(dat$val))))
-      dat$val <- sjlabelled::as_numeric(dat$val, keep.labels = F)
+      dat$val <- sjlabelled::as_numeric(dat$val, keep.labels = FALSE)
     else
       dat$val <- as.numeric(dat$val)
 
@@ -419,7 +476,7 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
     }
 
     colnames(dat2) <- c("val", "frq")
-    dat2$val <- sjlabelled::as_numeric(dat2$val, keep.labels = F)
+    dat2$val <- sjlabelled::as_numeric(dat2$val, keep.labels = FALSE)
 
     # join frq table and label columns
     mydat <- suppressMessages(dplyr::full_join(dat, dat2))
@@ -446,13 +503,41 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
 
     colnames(mydat) <- c("val", "frq")
 
+    if (!anyNA(suppressWarnings(as.numeric(attr(mydat$val, "levels"))))) {
+      mydat$val <- sjlabelled::as_numeric(mydat$val, keep.labels = FALSE)
+    }
+
     # add values as label
     mydat$label <- as.character("<none>")
+    mydat <- mydat[c("val", "label", "frq")]
+  }
+
+  min.frq.string <- sprintf("n < %g", min.frq)
+
+  if (any(mydat$frq[!is.na(mydat$val)] < min.frq)) {
+	  mydatS1 <- mydat[which(mydat$frq >= min.frq | is.na(mydat$val)), ]
+	  mydatS2 <- mydat[which(mydat$frq < min.frq & !is.na(mydat$val)), ]
+
+	  mydatS3 <- data_frame(
+      val = min.frq.string,
+      label = "<none>",
+      frq = sum(mydatS2$frq)
+    )
+
+	  if (mydatS3$frq == 0) {
+	    mydat <- mydatS1
+	  } else {
+  	  mydat <- rbind(mydatS1, mydatS3)
+  	  row.names(mydat) <- c(
+  	    row.names(mydat)[-length(row.names(mydat))],
+  	    as.character(as.integer(row.names(mydat)[length(row.names(mydat)) - 1]) + 1)
+  	  )
+	  }
   }
 
   # need numeric
   if (is.factor(x) || is.character(x)) {
-    x <- sjlabelled::as_numeric(x, keep.labels = F)
+    x <- sjlabelled::as_numeric(x, keep.labels = FALSE)
   }
 
   # check if we have any NA-values - if not, add row for NA's
@@ -470,16 +555,31 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
   # valid values are one row less, because last row is NA row
   valid.vals <- nrow(mydat) - 1
 
-  # sort categories ascending or descending
-  if (!is.null(sort.frq) && (sort.frq == "asc" || sort.frq == "desc")) {
-    ord <- order(mydat$frq[seq_len(valid.vals)], decreasing = (sort.frq == "desc"))
-    mydat <- mydat[c(ord, valid.vals + 1), ]
+  if (!all(is.na(mydat$val))) {
+
+    extra.vals <- 1
+    # Momentarily, in order to sort categories, we consider lower frequencies subtotal as a non valid value
+    if (is.na(mydat$val[valid.vals]) & mydat$val[valid.vals + 1] == min.frq.string) {
+      valid.vals <- valid.vals - 1
+      extra.vals <- 2
+    }
+
+
+    # sort categories ascending or descending
+    if (!is.null(sort.frq) && (sort.frq == "asc" || sort.frq == "desc")) {
+      ord <- order(mydat$frq[seq_len(valid.vals)], decreasing = (sort.frq == "desc"))
+    } else {
+      ord <- seq_len(valid.vals)
+    }
+    mydat <- mydat[c(ord, (valid.vals + extra.vals):(valid.vals + 1)), ]
   }
 
+  valid.vals <- nrow(mydat) - 1
+
   # raw percentages
   mydat$raw.prc <- mydat$frq / sum(mydat$frq)
 
-  # compute valud and cumulative percentages
+  # compute valid and cumulative percentages
   mydat$valid.prc <- c(mydat$frq[seq_len(valid.vals)] / sum(mydat$frq[seq_len(valid.vals)]), NA)
   mydat$cum.prc <- c(cumsum(mydat$valid.prc[seq_len(valid.vals)]), NA)
 
@@ -489,18 +589,35 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
   mydat$valid.prc <- 100 * round(mydat$valid.prc, 4)
 
   # "rename" labels for NA values
-  if (!is.null(mydat$label)) mydat$label[is.na(mydat$label)] <- "NA"
+  if (!is.null(mydat$label)) mydat$label[is.na(mydat$val)] <- NA_character_
 
-  # save original order
-  reihe <- sjlabelled::as_numeric(mydat$val, start.at = 1, keep.labels = F)
-
-  # sort
-  if (sort.frq == "none") mydat <- mydat[order(reihe), ]
+  if (!all(is.na(mydat$val))) {
+    if (extra.vals == 1) {
+      # save original order
+      reihe <- sjlabelled::as_numeric(mydat$val, start.at = 1, keep.labels = FALSE)
+      # sort
+      if (sort.frq == "none") mydat <- mydat[order(reihe), ]
+    } else if (extra.vals == 2) {
+      # save original order
+      reihe <- suppressWarnings(sjlabelled::as_numeric(mydat$val[-c(valid.vals, valid.vals + 1)], start.at = 1, keep.labels = FALSE))
+      # sort
+      if (sort.frq == "none") mydat <- mydat[c(order(reihe), valid.vals, valid.vals + 1), ]
+    }
+  }
 
   # remove NA, if requested
   has.na <- mydat$frq[nrow(mydat)] > 0
-  if ((!is.logical(show.na) && show.na == "auto" && !has.na) || identical(show.na, FALSE))
+  if ((!is.logical(show.na) && show.na == "auto" && !has.na) || identical(show.na, FALSE)) {
     mydat <- mydat[-nrow(mydat), ]
+  }
+
+  # compute relative confidence intervals
+  total_n <- sum(mydat$frq)
+  rel_frq <- as.numeric(mydat$frq / total_n)
+  ci <- stats::qnorm(.975) * suppressWarnings(sqrt(rel_frq * (1 - rel_frq) / total_n))
+  total_ci <- data.frame(lower = total_n * (rel_frq - ci), upper = total_n * (rel_frq + ci))
+  relative_ci <- data.frame(lower = rel_frq - ci, upper = rel_frq + ci)
+
 
   # add variable label and type as attribute, for print-method
 
@@ -516,19 +633,32 @@ frq_helper <- function(x, sort.frq, weight.by, cn, auto.grp, title = NULL, show.
   attr(mydat, "mean") <- mean.value
   attr(mydat, "sd") <- sd.value
 
+  attr(mydat, "ci") <- total_ci
+  attr(mydat, "relative.ci") <- relative_ci
+
+  row.names(mydat) <- NULL
+
   mydat
 }
 
 
-get_grouped_title <- function(x, grps, i, sep = "\n") {
+get_grouped_title <- function(x, grps, i, sep = ", ", long = FALSE) {
   # create title for first grouping level
   tp <- get_title_part(x, grps, 1, i)
-  title <- sprintf("%s: %s", tp[1], tp[2])
+
+  if (long)
+    title <- sprintf("%s: %s", tp[1], tp[2])
+  else
+    title <- sprintf("%s", tp[2])
 
   # do we have another groupng variable?
-  if (length(attr(x, "vars", exact = T)) > 1) {
+  if (length(dplyr::group_vars(x)) > 1) {
     tp <- get_title_part(x, grps, 2, i)
-    title <- sprintf("%s%s%s: %s", title, sep, tp[1], tp[2])
+
+    if (long)
+      title <- sprintf("%s%s%s: %s", title, sep, tp[1], tp[2])
+    else
+      title <- sprintf("%s%s%s", title, sep, tp[2])
   }
 
   # return title
@@ -542,35 +672,33 @@ get_title_part <- function(x, grps, level, i) {
 
   # get values from value labels
   vals <- sjlabelled::get_values(x[[var.name]])
+  t2 <- NULL
 
   # if we have no value labels, get values directly
   if (is.null(vals)) {
-    vals <- unique(x[[var.name]])
+    vals <- grps[[var.name]]
+    if (is.factor(grps[[var.name]])) vals <- as.character(vals)
     lab.pos <- i
   } else {
     # find position of value labels for current group
     lab.pos <- which(vals == grps[[var.name]][i])
+    t2 <- sjlabelled::get_labels(x[[var.name]])[lab.pos]
   }
 
   # get variable and value labels
   t1 <- sjlabelled::get_label(x[[var.name]], def.value = var.name)
-  t2 <- sjlabelled::get_labels(x[[var.name]])[lab.pos]
 
   # if we have no value label, use value instead
-  if (is.null(t2)) t2 <- vals[lab.pos]
+  if (sjmisc::is_empty(t2)) t2 <- vals[lab.pos]
 
   # generate title
   c(t1, t2)
 }
 
 
-#' @importFrom tidyr nest
-#' @importFrom dplyr filter
-#' @importFrom stats complete.cases
-#' @importFrom rlang .data
 get_grouped_data <- function(x) {
   # nest data frame
-  grps <- tidyr::nest(x)
+  grps <- .nest(x)
 
   # remove NA category for grouped data
   cc <- grps %>%
@@ -582,7 +710,7 @@ get_grouped_data <- function(x) {
 
   # arrange data
 
-  if (length(attr(x, "vars", exact = T)) == 1)
+  if (length(dplyr::group_vars(x)) == 1)
     reihe <- order(grps[[1]])
   else
     reihe <- order(grps[[1]], grps[[2]])
@@ -594,3 +722,15 @@ get_grouped_data <- function(x) {
 
 
 no_character <- function(x) !is.character(x)
+
+
+#' @export
+as.data.frame.sjmisc_frq <- function(x, row.names = NULL, optional = FALSE, ...) {
+  x <- lapply(x, function(i) {
+    i$variable <- attr(i, "label")
+    i$group <- attr(i, "group")
+    cols <- c("variable", "group", "val", "label", "frq", "raw.prc", "vaid.prc", "cum.prc")
+    i[, intersect(cols, colnames(i))]
+  })
+  do.call(rbind, x)
+}
