@@ -1,4 +1,3 @@
-#' @importFrom dplyr group_by
 prepare_mids_recode <- function(x) {
   # check if suggested package is available
   if (!requireNamespace("mice", quietly = TRUE))
@@ -6,7 +5,7 @@ prepare_mids_recode <- function(x) {
 
   # check classes
   if (!inherits(x, "mids"))
-    stop("`x` must be a `mids`-object, as returned by the `mice()`-function.", call. = F)
+    stop("`x` must be a `mids`-object, as returned by the `mice()`-function.", call. = FALSE)
 
 
   # convert mids into long-data.frame
@@ -22,7 +21,6 @@ prepare_mids_recode <- function(x) {
 }
 
 
-#' @importFrom purrr map
 final_mids_recode <- function(x) {
   # check if suggested package is available
 
